class Point {
  constructor(canvas, options) {
    this.canvas = canvas;
    this.ctx = canvas.getContext("2d");
    options = options || {};
    this.options = {
      pointColor: options.pointColor || "rgba(255,255,255,1)", // 点的颜色
    };

    this.x = this.randomFloat(0, this.canvas.width); // 随机x坐标
    this.y = this.randomFloat(0, this.canvas.height); // 随机y坐标

    var speed = this.randomFloat(0.3, 1.4); // 随机速度
    var angle = this.randomFloat(0, 2*Math.PI); // 随机角度

    this.dx = Math.sin(angle) * speed; // 计算x轴位移分量“嘚儿塔踢”
    this.dy = Math.cos(angle) * speed; // 计算y轴位移分量“嘚儿塔踢”

    this.r = this.randomFloat(1, 1.8); // 随机点大小

    this.color = this.options.pointColor;
	
	this.dpageScrollDistance = 0; // 页面滚动距离的变化量
    this.pageScrollDistance = 0; // 目前页面滚动距离
	this.pageScrollDistanceOld = 0; // 上一次页面滚动距离
    document.addEventListener("scroll", (event) => { // 监听页面滚动事件
      this.pageScrollDistance = window.scrollY;
    });
  }
  
  move() {
    this.x += this.dx; // x坐标增加x方向位移
    if (this.x < -100) { // 超出屏幕边缘-100的距离后
      this.x = this.canvas.width;
    } else if (this.x > this.canvas.width+100) {
      this.x = 0;
    }
    this.y += this.dy; // y坐标增加y方向位移
	this.dpageScrollDistance = this.pageScrollDistance - this.pageScrollDistanceOld;
    this.pageScrollDistanceOld = this.pageScrollDistance;
	this.y -= this.dpageScrollDistance/5; // 跟着页面进行滚动，除以5是为了有交错感
    if (this.y < -100) { // 超出屏幕边缘-100的距离后
      this.y = this.y + this.canvas.height + 200;
    } else if (this.y > this.canvas.height+100) {
      this.y = this.y - this.canvas.height - 200;
    }
  }

  draw() { // 绘制每一个粒子
    this.ctx.fillStyle = this.color;
    this.ctx.beginPath();
    this.ctx.arc(this.x, this.y, this.r, 0, Math.PI * 2);
    this.ctx.closePath();
    this.ctx.fill();
  }

  randomInt(min, max) { // 返回随机整数
    return Math.floor((max - min + 1) * Math.random() + min);
  }

  randomFloat(min, max) { // 返回随机小数
    return (max - min) * Math.random() + min;
  }
}

class ParticleBackground {
  constructor(options) {
    options = options || {};
    this.canvas = document.createElement("canvas");
    this.ctx = this.canvas.getContext("2d");
    this.points = [];

    this.degree = 2.5; // 不知道这行有啥用
    this.startTime = new Date().getTime(); // 我还是不知道这行有啥用

    this.options = {
      backgroundColor: options.backgroundColor || "rgba(0, 0, 0,1)", // 背景颜色
      pointNum: options.pointNum || 150, // 粒子数量
      pointColor: options.pointColor || "rgba(255,255,255,1)", // 粒子颜色
      lineLength: options.lineLength || 10000, // 连线长度（实际上是长度的平方，这样在用勾股定理计算时方便点）
    };

    this.p0 = new Point(this.canvas, this.options);
    this.p0.dx = this.p0.dy = 0;

    this.canvas.width = window.innerWidth; // 设置画布宽度
    this.canvas.height = window.innerHeight; // 设置画布高度
    this.canvas.style.cssText = `
      position: fixed;
      top: 0px;
      left: 0px;
      z-index: -1;
      opacity: 1.0;
    `; // 设置画布样式
    document.body.appendChild(this.canvas, this.options);
  }

  initPoints(num) {
    if (!num) num = this.options.pointNum;
    for (var i = 0; i < num; ++i) {
      this.points.push(new Point(this.canvas));
    }
    this.initListner();
  }

  drawLine(p1, p2, deg, p2dx, p2dy) {
    var dx = p1.x - p2.x;
    var dy = p1.y - p2.y;
    var dis2 = dx * dx + dy * dy;

	var P2Angel = Math.atan2(p2dy, p2dx);
	var LineAngel = Math.atan2(dy, dx);
    var angle = Math.abs(P2Angel - LineAngel);
	if (angle > Math.PI) {
		angle = 2 * Math.PI - angle;
	}
    if (dis2 < 2 * this.options.lineLength) {
      if (dis2 > this.options.lineLength/5 && angle < Math.PI / 2) {
        if (p1 === this.p0) {
          p2.x += dx * 0.025;
          p2.y += dy * 0.025;
        } else return;
      }
      var t = (1.05 - dis2 / this.options.lineLength) * 0.2 * deg;
      this.ctx.strokeStyle = "rgba(255,255,255," + t + ")"; // 绘制线条
      this.ctx.beginPath();
      this.ctx.lineWidth = 1.5;
      this.ctx.moveTo(p1.x, p1.y);
      this.ctx.lineTo(p2.x, p2.y);
      this.ctx.closePath();
      this.ctx.stroke();
    }
    return;
  }

  drawFrame() {
    this.canvas.width = window.innerWidth;
    this.canvas.height = window.innerHeight;
    this.ctx.fillStyle = this.options.backgroundColor;
    this.ctx.fillRect(0, 0, this.canvas.width, this.canvas.height);

    var arr = this.p0.x == null ? this.points : [this.p0].concat(this.points); // 如果鼠标所在点不存在，则绘制所有点的连线，否则绘制鼠标点和其他点的连线
    for (var i = 0; i < arr.length; ++i) {
      for (var j = i + 1; j < arr.length; ++j) {
        this.drawLine(arr[i], arr[j], 1.0, arr[j].dx, arr[j].dy); // 依次绘制点的连线
      }
      arr[i].draw();
      arr[i].move();
    }

    window.requestAnimationFrame(this.drawFrame.bind(this));
  }

  initListner() {
    const _this = this;
    document.onmousemove = function (ev) {
      _this.p0.x = ev.clientX;
      _this.p0.y = ev.clientY;
    };
    document.onmousedown = function (ev) {
      _this.degree = 5.0;
      _this.p0.x = ev.clientX;
      _this.p0.y = ev.clientY;
    };
    document.onmouseup = function (ev) {
      _this.degree = 2.5;
      _this.p0.x = ev.clientX;
      _this.p0.y = ev.clientY;
    };
    window.onmouseout = function () {
      _this.p0.x = null;
      _this.p0.y = null;
    };
  }
}
